@@ -18,7 +18,6 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.oldresult;
 
 import org.gradle.api.artifacts.ModuleDependency;
 import org.gradle.api.artifacts.ResolvedArtifact;
-import org.gradle.api.internal.artifacts.ResolvedConfigurationIdentifier;
 
 import java.util.Set;
 
@@ -26,5 +25,5 @@ public interface ResolvedContentsMapping {
 
     Set<ResolvedArtifact> getArtifacts(long id);
 
-    ModuleDependency getModuleDependency(ResolvedConfigurationIdentifier id);
+    ModuleDependency getModuleDependency(long id);
 }
