@@ -49,26 +49,43 @@ public class ComponentSelectorSerializer implements Serializer<ComponentSelector
             throw new IllegalArgumentException("Provided component selector may not be null");
         }
 
-        if (value instanceof DefaultModuleComponentSelector) {
+        Implementation implementation = resolveImplementation(value);
+
+        encoder.writeByte(implementation.getId());
+
+        if (implementation == Implementation.MODULE) {
             ModuleComponentSelector moduleComponentSelector = (ModuleComponentSelector) value;
-            encoder.writeByte(Implementation.MODULE.getId());
             encoder.writeString(moduleComponentSelector.getGroup());
             encoder.writeString(moduleComponentSelector.getModule());
             encoder.writeString(moduleComponentSelector.getVersion());
-        } else if (value instanceof DefaultProjectComponentSelector) {
+        } else if (implementation == Implementation.BUILD) {
             ProjectComponentSelector projectComponentSelector = (ProjectComponentSelector) value;
-            encoder.writeByte(Implementation.BUILD.getId());
             encoder.writeString(projectComponentSelector.getBuildName());
             encoder.writeString(projectComponentSelector.getProjectPath());
-        } else if (value instanceof DefaultLibraryComponentSelector) {
+        } else if (implementation == Implementation.LIBRARY) {
             LibraryComponentSelector libraryComponentSelector = (LibraryComponentSelector) value;
-            encoder.writeByte(Implementation.LIBRARY.getId());
             encoder.writeString(libraryComponentSelector.getProjectPath());
             encoder.writeNullableString(libraryComponentSelector.getLibraryName());
             encoder.writeNullableString(libraryComponentSelector.getVariant());
+        } else {
+            throw new IllegalStateException("Unsupported implementation type: " + implementation);
+        }
+    }
+
+    private ComponentSelectorSerializer.Implementation resolveImplementation(ComponentSelector value) {
+        Implementation implementation;
+
+        if (value instanceof ModuleComponentSelector) {
+            implementation = Implementation.MODULE;
+        } else if (value instanceof ProjectComponentSelector) {
+            implementation = Implementation.BUILD;
+        } else if (value instanceof LibraryComponentSelector) {
+            implementation = Implementation.LIBRARY;
         } else {
             throw new IllegalArgumentException("Unsupported component selector class: " + value.getClass());
         }
+
+        return implementation;
     }
 
     private enum Implementation {
