@@ -18,7 +18,7 @@ package org.gradle.plugins.ide.internal.tooling;
 
 import org.gradle.api.JavaVersion;
 import org.gradle.api.Project;
-import org.gradle.api.internal.artifacts.ivyservice.projectmodule.CompositeProjectDirectoryMapper;
+import org.gradle.api.internal.composite.CompositeBuildIdeProjectResolver;
 import org.gradle.api.plugins.JavaPluginConvention;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.plugins.ide.idea.IdeaPlugin;
@@ -57,13 +57,13 @@ import java.util.Set;
 
 public class IdeaModelBuilder implements ToolingModelBuilder {
     private final GradleProjectBuilder gradleProjectBuilder;
-    private final CompositeProjectDirectoryMapper compositeProjectMapper;
+    private final CompositeBuildIdeProjectResolver compositeProjectMapper;
 
     private boolean offlineDependencyResolution;
 
     public IdeaModelBuilder(GradleProjectBuilder gradleProjectBuilder, ServiceRegistry services) {
         this.gradleProjectBuilder = gradleProjectBuilder;
-        compositeProjectMapper = new CompositeProjectDirectoryMapper(services);
+        compositeProjectMapper = new CompositeBuildIdeProjectResolver(services);
     }
 
     @Override
@@ -84,7 +84,7 @@ public class IdeaModelBuilder implements ToolingModelBuilder {
         for (Project p : allProjects) {
             p.getPluginManager().apply(IdeaPlugin.class);
         }
-        ideaPluginFor(root).makeSureModuleNamesAreUnique();
+        ideaPluginFor(root).performPostEvaluationActions();
     }
 
     private DefaultIdeaProject build(Project project, DefaultGradleProject rootGradleProject) {
@@ -125,7 +125,7 @@ public class IdeaModelBuilder implements ToolingModelBuilder {
         for (Dependency dependency : resolved) {
             if (dependency instanceof SingleEntryModuleLibrary) {
                 SingleEntryModuleLibrary d = (SingleEntryModuleLibrary) dependency;
-                DefaultIdeaSingleEntryLibraryDependency defaultDependency = new org.gradle.tooling.internal.idea.DefaultIdeaSingleEntryLibraryDependency()
+                DefaultIdeaSingleEntryLibraryDependency defaultDependency = new DefaultIdeaSingleEntryLibraryDependency()
                     .setFile(d.getLibraryFile())
                     .setSource(d.getSourceFile())
                     .setJavadoc(d.getJavadocFile())
@@ -137,17 +137,17 @@ public class IdeaModelBuilder implements ToolingModelBuilder {
                 }
                 dependencies.add(defaultDependency);
             } else if (dependency instanceof ModuleDependency) {
-                ModuleDependency d = (ModuleDependency) dependency;
-                DefaultIdeaModule targetModule = modules.get(d.getName());
-                File targetProjectDirectory = targetModule == null
-                    ? compositeProjectMapper.transform(d.getGradlePath())
-                    : targetModule.getGradleProject().getProjectDirectory();
-                DefaultIdeaModuleDependency defaultDependency = new org.gradle.tooling.internal.idea.DefaultIdeaModuleDependency()
-                    .setExported(d.isExported())
-                    .setScope(new DefaultIdeaDependencyScope(d.getScope()))
-                    .setDependencyModule(targetModule)
-                    .setProjectDirectory(targetProjectDirectory);
-                dependencies.add(defaultDependency);
+                ModuleDependency moduleDependency = (ModuleDependency) dependency;
+
+                DefaultIdeaModuleDependency ideaModuleDependency = new DefaultIdeaModuleDependency(moduleDependency.getName())
+                    .setExported(moduleDependency.isExported())
+                    .setScope(new DefaultIdeaDependencyScope(moduleDependency.getScope()));
+
+                // Find IdeaModule model for dependency within same build: may be null
+                DefaultIdeaModule targetModule = modules.get(moduleDependency.getName());
+                ideaModuleDependency.setDependencyModule(targetModule);
+
+                dependencies.add(ideaModuleDependency);
             }
         }
         modules.get(ideaModule.getName()).setDependencies(dependencies);
