@@ -19,6 +19,8 @@ import com.google.common.base.Function;
 import com.google.common.base.Predicate;
 import com.google.common.collect.FluentIterable;
 import com.google.common.collect.Iterables;
+import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import net.rubygrapefruit.platform.SystemInfo;
 import net.rubygrapefruit.platform.WindowsRegistry;
 import org.gradle.api.JavaVersion;
@@ -39,6 +41,7 @@ import org.gradle.util.CollectionUtils;
 
 import java.io.File;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
@@ -75,10 +78,20 @@ public abstract class AvailableJavaHomes {
      * Returns a JDK for each of the given java versions, if available.
      */
     public static List<Jvm> getJdks(final String... versions) {
-        final Set<JavaVersion> remaining = new HashSet<JavaVersion>();
-        for (String version : versions) {
-            remaining.add(JavaVersion.toVersion(version));
-        }
+        List<JavaVersion> javaVersions = Lists.transform(Arrays.asList(versions), new Function<String, JavaVersion>() {
+            @Override
+            public JavaVersion apply(@javax.annotation.Nullable String version) {
+                return JavaVersion.toVersion(version);
+            }
+        });
+        return getJdks(Iterables.toArray(javaVersions, JavaVersion.class));
+    }
+
+    /**
+     * Returns a JDK for each of the given java versions, if available.
+     */
+    public static List<Jvm> getJdks(JavaVersion... versions) {
+        final Set<JavaVersion> remaining = Sets.newHashSet(versions);
         return getAvailableJdks(new Spec<JvmInstallation>() {
             @Override
             public boolean isSatisfiedBy(JvmInstallation element) {
