@@ -15,7 +15,6 @@
  */
 package org.gradle.testing.jacoco.plugins;
 
-import groovy.lang.GroovyObjectSupport;
 import org.gradle.api.Action;
 import org.gradle.api.Incubating;
 import org.gradle.api.Project;
@@ -35,7 +34,7 @@ import java.util.concurrent.Callable;
  * Extension including common properties and methods for Jacoco.
  */
 @Incubating
-public class JacocoPluginExtension extends GroovyObjectSupport {
+public class JacocoPluginExtension {
 
     public static final String TASK_EXTENSION_NAME = "jacoco";
 
@@ -43,7 +42,7 @@ public class JacocoPluginExtension extends GroovyObjectSupport {
     protected final Project project;
     private final JacocoAgentJar agent;
 
-    private String toolVersion = "0.7.6.201602180812";
+    private String toolVersion = "0.7.7.201606060606";
     private File reportsDir;
 
     /**
@@ -139,8 +138,4 @@ public class JacocoPluginExtension extends GroovyObjectSupport {
     public void setLogger(Logger logger) {
         this.logger = logger;
     }
-
-    public static String getTASK_EXTENSION_NAME() {
-        return TASK_EXTENSION_NAME;
-    }
 }
