@@ -26,6 +26,7 @@ import org.gradle.internal.classpath.ClassPath;
 import org.gradle.internal.classpath.DefaultClassPath;
 import org.gradle.internal.installation.CurrentGradleInstallation;
 import org.gradle.internal.installation.GradleInstallation;
+import org.gradle.internal.io.StreamByteBuffer;
 import org.gradle.internal.serialize.FlushableEncoder;
 import org.gradle.internal.serialize.kryo.KryoBackedEncoder;
 import org.gradle.launcher.daemon.DaemonExecHandleBuilder;
@@ -42,8 +43,6 @@ import org.gradle.util.CollectionUtils;
 import org.gradle.util.GFileUtils;
 import org.gradle.util.GradleVersion;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
@@ -110,8 +109,8 @@ public class DefaultDaemonStarter implements DaemonStarter {
         daemonArgs.add(GradleVersion.current().getVersion());
 
         // Serialize configuration to daemon via the process' stdin
-        ByteArrayOutputStream serializedConfig = new ByteArrayOutputStream();
-        FlushableEncoder encoder = new KryoBackedEncoder(new EncodedStream.EncodedOutput(serializedConfig));
+        StreamByteBuffer buffer = new StreamByteBuffer();
+        FlushableEncoder encoder = new KryoBackedEncoder(new EncodedStream.EncodedOutput(buffer.getOutputStream()));
         try {
             encoder.writeString(daemonParameters.getGradleUserHomeDir().getAbsolutePath());
             encoder.writeString(daemonDir.getBaseDir().getAbsolutePath());
@@ -130,7 +129,7 @@ public class DefaultDaemonStarter implements DaemonStarter {
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
-        ByteArrayInputStream stdInput = new ByteArrayInputStream(serializedConfig.toByteArray());
+        InputStream stdInput = buffer.getInputStream();
 
         return startProcess(daemonArgs, daemonDir.getVersionedDir(), stdInput);
     }
