@@ -16,7 +16,13 @@
 package org.gradle.nativeplatform.plugins;
 
 import org.apache.commons.lang.StringUtils;
-import org.gradle.api.*;
+import org.gradle.api.Action;
+import org.gradle.api.DefaultTask;
+import org.gradle.api.Incubating;
+import org.gradle.api.NamedDomainObjectFactory;
+import org.gradle.api.Namer;
+import org.gradle.api.Plugin;
+import org.gradle.api.Task;
 import org.gradle.api.artifacts.repositories.ArtifactRepository;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.file.SourceDirectorySet;
@@ -24,8 +30,11 @@ import org.gradle.api.internal.DefaultPolymorphicDomainObjectContainer;
 import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.SourceDirectorySetFactory;
 import org.gradle.api.internal.project.ProjectInternal;
+import org.gradle.api.internal.project.ProjectRegistry;
+import org.gradle.api.internal.resolve.ProjectModelResolver;
 import org.gradle.api.plugins.ExtensionContainer;
 import org.gradle.api.tasks.TaskContainer;
+import org.gradle.internal.Cast;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.language.base.LanguageSourceSet;
@@ -36,9 +45,43 @@ import org.gradle.language.base.plugins.ComponentModelBasePlugin;
 import org.gradle.language.nativeplatform.DependentSourceSet;
 import org.gradle.language.nativeplatform.HeaderExportingSourceSet;
 import org.gradle.language.nativeplatform.internal.DependentSourceSetInternal;
-import org.gradle.model.*;
-import org.gradle.nativeplatform.*;
-import org.gradle.nativeplatform.internal.*;
+import org.gradle.model.Defaults;
+import org.gradle.model.Each;
+import org.gradle.model.Finalize;
+import org.gradle.model.Model;
+import org.gradle.model.ModelMap;
+import org.gradle.model.Mutate;
+import org.gradle.model.Path;
+import org.gradle.model.RuleSource;
+import org.gradle.nativeplatform.BuildTypeContainer;
+import org.gradle.nativeplatform.FlavorContainer;
+import org.gradle.nativeplatform.NativeComponentSpec;
+import org.gradle.nativeplatform.NativeDependencySet;
+import org.gradle.nativeplatform.NativeExecutableBinarySpec;
+import org.gradle.nativeplatform.NativeExecutableSpec;
+import org.gradle.nativeplatform.NativeLibrarySpec;
+import org.gradle.nativeplatform.PrebuiltLibraries;
+import org.gradle.nativeplatform.PrebuiltLibrary;
+import org.gradle.nativeplatform.Repositories;
+import org.gradle.nativeplatform.SharedLibraryBinarySpec;
+import org.gradle.nativeplatform.StaticLibraryBinarySpec;
+import org.gradle.nativeplatform.TargetedNativeComponent;
+import org.gradle.nativeplatform.internal.DefaultBuildTypeContainer;
+import org.gradle.nativeplatform.internal.DefaultFlavor;
+import org.gradle.nativeplatform.internal.DefaultFlavorContainer;
+import org.gradle.nativeplatform.internal.DefaultNativeExecutableBinarySpec;
+import org.gradle.nativeplatform.internal.DefaultNativeExecutableSpec;
+import org.gradle.nativeplatform.internal.DefaultNativeLibrarySpec;
+import org.gradle.nativeplatform.internal.DefaultSharedLibraryBinarySpec;
+import org.gradle.nativeplatform.internal.DefaultStaticLibraryBinarySpec;
+import org.gradle.nativeplatform.internal.NativeBinarySpecInternal;
+import org.gradle.nativeplatform.internal.NativeComponents;
+import org.gradle.nativeplatform.internal.NativeDependentBinariesResolutionStrategy;
+import org.gradle.nativeplatform.internal.NativeExecutableBinarySpecInternal;
+import org.gradle.nativeplatform.internal.NativePlatformResolver;
+import org.gradle.nativeplatform.internal.SharedLibraryBinarySpecInternal;
+import org.gradle.nativeplatform.internal.StaticLibraryBinarySpecInternal;
+import org.gradle.nativeplatform.internal.TargetedNativeComponentInternal;
 import org.gradle.nativeplatform.internal.configure.NativeComponentRules;
 import org.gradle.nativeplatform.internal.pch.PchEnabledLanguageTransform;
 import org.gradle.nativeplatform.internal.prebuilt.DefaultPrebuiltLibraries;
@@ -52,9 +95,17 @@ import org.gradle.nativeplatform.tasks.LinkSharedLibrary;
 import org.gradle.nativeplatform.tasks.PrefixHeaderFileGenerateTask;
 import org.gradle.nativeplatform.toolchain.internal.DefaultNativeToolChainRegistry;
 import org.gradle.nativeplatform.toolchain.internal.NativeToolChainRegistryInternal;
-import org.gradle.platform.base.*;
+import org.gradle.platform.base.BinaryContainer;
+import org.gradle.platform.base.BinaryTasks;
+import org.gradle.platform.base.ComponentSpecContainer;
+import org.gradle.platform.base.ComponentType;
+import org.gradle.platform.base.Platform;
+import org.gradle.platform.base.PlatformContainer;
+import org.gradle.platform.base.SourceComponentSpec;
+import org.gradle.platform.base.TypeBuilder;
 import org.gradle.platform.base.internal.HasIntermediateOutputsComponentSpec;
 import org.gradle.platform.base.internal.PlatformResolvers;
+import org.gradle.platform.base.internal.dependents.DependentBinariesResolver;
 
 import javax.inject.Inject;
 import java.io.File;
@@ -201,7 +252,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
                 @Override
                 public void execute(DependentSourceSetInternal dependentSourceSet) {
                     if (dependentSourceSet.getPreCompiledHeader() != null) {
-                        String prefixHeaderDirName = String.format("tmp/%s/%s/prefixHeaders", componentSpec.getName(), dependentSourceSet.getName());
+                        String prefixHeaderDirName = "tmp/" + componentSpec.getName() + "/" + dependentSourceSet.getName() + "/prefixHeaders";
                         File prefixHeaderDir = new File(buildDir, prefixHeaderDirName);
                         File prefixHeaderFile = new File(prefixHeaderDir, "prefix-headers.h");
                         dependentSourceSet.setPrefixHeaderFile(prefixHeaderFile);
@@ -215,7 +266,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             for (final SourceComponentSpec nativeComponentSpec : components.withType(SourceComponentSpec.class).values()) {
                 for (final DependentSourceSetInternal dependentSourceSet : nativeComponentSpec.getSources().withType(DependentSourceSetInternal.class).values()) {
                     if (dependentSourceSet.getPrefixHeaderFile() != null) {
-                        String taskName = String.format("generate%s%sPrefixHeaderFile", StringUtils.capitalize(nativeComponentSpec.getName()), StringUtils.capitalize(dependentSourceSet.getName()));
+                        String taskName = "generate" + StringUtils.capitalize(nativeComponentSpec.getName()) + StringUtils.capitalize(dependentSourceSet.getName()) + "PrefixHeaderFile";
                         tasks.create(taskName, PrefixHeaderFileGenerateTask.class, new Action<PrefixHeaderFileGenerateTask>() {
                             @Override
                             public void execute(PrefixHeaderFileGenerateTask prefixHeaderFileGenerateTask) {
@@ -239,7 +290,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
                             if (dependentSourceSet.getPreCompiledHeader() != null) {
                                 nativeBinarySpec.addPreCompiledHeaderFor(dependentSourceSet);
                                 final SourceTransformTaskConfig pchTransformTaskConfig = transform.getPchTransformTask();
-                                String pchTaskName = String.format("%s%s%sPreCompiledHeader", pchTransformTaskConfig.getTaskPrefix(), StringUtils.capitalize(nativeBinarySpec.getProjectScopedName()), StringUtils.capitalize(dependentSourceSet.getName()));
+                                String pchTaskName = pchTransformTaskConfig.getTaskPrefix() + StringUtils.capitalize(nativeBinarySpec.getProjectScopedName()) + StringUtils.capitalize(dependentSourceSet.getName()) + "PreCompiledHeader";
                                 Task pchTask = tasks.create(pchTaskName, pchTransformTaskConfig.getTaskType(), new Action<DefaultTask>() {
                                     @Override
                                     public void execute(DefaultTask task) {
@@ -304,6 +355,21 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             NativeComponents.createExecutableTask(executableBinary, executableBinary.getExecutable().getFile());
         }
 
+        @Defaults
+        public void createBuildDependentComponentsTasks(ModelMap<Task> tasks, ComponentSpecContainer components, BinaryContainer binaries) {
+            NativeComponents.createBuildDependentComponentsTasks(tasks, components);
+        }
+
+        @BinaryTasks
+        public void createBuildDependentBinariesTasks(ModelMap<Task> tasks, NativeBinarySpecInternal nativeBinary) {
+            NativeComponents.createBuildDependentBinariesTasks(nativeBinary, nativeBinary.getNamingScheme());
+        }
+
+        @Finalize
+        public void wireBuildDependentTasks(ModelMap<Task> tasks, BinaryContainer binaries, DependentBinariesResolver dependentsResolver, ServiceRegistry serviceRegistry) {
+            NativeComponents.wireBuildDependentTasks(tasks, binaries, dependentsResolver, serviceRegistry.get(ProjectModelResolver.class));
+        }
+
         /**
          * Can't use @BinaryTasks because the binary is not _built-by_ the install task, but it is associated with it. Rule is called multiple times, so need to check for task existence before
          * creating.
@@ -318,11 +384,11 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         @Finalize
         void applyHeaderSourceSetConventions(@Each HeaderExportingSourceSet headerSourceSet) {
             // Only apply default locations when none explicitly configured
-            if (headerSourceSet.getExportedHeaders().getSrcDirs().isEmpty()) {
-                headerSourceSet.getExportedHeaders().srcDir(String.format("src/%s/headers", headerSourceSet.getParentName()));
+            if (headerSourceSet.getExportedHeaders().getSourceDirectories().isEmpty()) {
+                headerSourceSet.getExportedHeaders().srcDir("src/" + headerSourceSet.getParentName() + "/headers");
             }
 
-            headerSourceSet.getImplicitHeaders().setSrcDirs(headerSourceSet.getSource().getSrcDirs());
+            headerSourceSet.getImplicitHeaders().setSrcDirs(headerSourceSet.getSource().getSourceDirectories());
             headerSourceSet.getImplicitHeaders().include("**/*.h");
         }
 
@@ -338,6 +404,13 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             FileCollectionFactory fileCollectionFactory = serviceRegistry.get(FileCollectionFactory.class);
             NativeComponentRules.createBinariesImpl(nativeComponent, platforms, buildTypes, flavors, nativePlatforms, nativeDependencyResolver, fileCollectionFactory);
         }
+
+        @Defaults
+        void registerNativeDependentBinariesResolutionStrategy(DependentBinariesResolver resolver, ServiceRegistry serviceRegistry) {
+            ProjectRegistry<ProjectInternal> projectRegistry = Cast.uncheckedCast(serviceRegistry.get(ProjectRegistry.class));
+            ProjectModelResolver projectModelResolver = serviceRegistry.get(ProjectModelResolver.class);
+            resolver.register(new NativeDependentBinariesResolutionStrategy(projectRegistry, projectModelResolver));
+        }
     }
 
     private static class DefaultRepositories extends DefaultPolymorphicDomainObjectContainer<ArtifactRepository> implements Repositories {
